-module(dir).
-compile(export_all).
-author('kapolnai@iit.bme.hu, modified by: szeredi@cs.bme.hu').
-vsn('$LastChangedDate: 2013-11-03 11:45:36 +0100 (Sun, 03 Nov 2013) $$').
-define(Sep, $/).

% @type tree()      = file() | directory().
% @type file()      = {f, name()}.
% @type directory() = {d, name(), [tree()]}.
% @type name()      = string().

%% @spec basename(string()) -> name().
%% Path string utolsó ?Sep karaktere utáni szuffixum.
%basename(Path) ->
%    lists:nthtail(string:rchr(Path, ?Sep), Path).

% @spec explore(string()) -> tree().
% Helyi, Path gyökerű fájlrendszer alapján épít fel egy fát.
explore(Path)->
    Name = filename:basename(Path),
    case file:list_dir(Path) of
        {ok, Content} ->
            {d, Name, [explore(Path ++ [?Sep|Item]) || Item <- Content]};
        {error, _} ->
            {f, Name}
    end.

% Fa mérete (könyvtárak és fájlok száma).
count({f,_}) ->
    1;
count({d,_,L}) ->
    1 + lists:sum([count(I) || I <- L]).

% @spec subtree(string(), tree()) -> tree().
% Tree fa Path útvonalú elérhető részfája.
subtree_by_path(Path, Tree) ->
    Tokens = string:tokens(Path, [?Sep]),
    subtree(Tokens, Tree).

% @spec subtree(Path::[name()], Tree::tree()) -> tree() | notfound.
% Tree fa Path útvonalon található részfája.
subtree([Name], {f,Name} = Tree) ->
    Tree;
subtree([Name], {d,Name,_} = Tree) ->
    Tree;
subtree([Root|Path], {d,Root,L}) ->
    [Sub|_] = Path,
    case lists:keyfind(Sub, 2, L) of
        false   -> notfound;
        SubTree -> subtree(Path, SubTree)
    end;
subtree(_, _) ->
    notfound.

% @spec whereis(name(), tree()) -> string().
% A Name fájl elérési útvonala a T fában.
whereis(Name, T) ->
    whereis(Name, T, []).

% @spec whereis(name(), tree(), Path::string()) -> Path1::string().
% Path1 a Name fájl elérési útvonala a T fában Path után fűzve.
whereis(Name, {d,Name,_}, Path) ->
    [Path ++ Name];
whereis(Name, {f,Name}, Path) ->
    [Path ++ Name];
whereis(_Name, {f,_}, _) ->
    [];
whereis(Name, {d,Dirname,L}, Path0) ->
    [Path1 || SubTree <- L,
              Path1 <- whereis(Name, SubTree, Path0 ++ Dirname ++ [?Sep])
%              io:format("-~p,~p:~p~n", [Path,Dirname,Path2])=:=ok,
    ].

test(1) ->
    explore("home");
test(2) ->
    Home = explore("home"),
    PathToks = string:tokens("home/kitti/.firefox", "/"),
    dir:subtree(PathToks, Home);
test(3) ->
    Home = explore("home"),
    PathToks = string:tokens("home/kitti/firefox", "/"),
    dir:subtree(PathToks, Home).

